@@ -10,7 +10,6 @@ const char *ripe_servers[] = {
     "whois.ripe.net",
     "whois.apnic.net",
     "whois.afrinic.net",
-    "whois.denic.de",
     "rr.arin.net",		/* does not accept the old syntax */
     "whois.6bone.net",		/* 3.0.0b1 */
     "whois.connect.com.au",	/* 3.0.0b1 */
@@ -25,13 +24,12 @@ const char *ripe_servers[] = {
     "whois.nic.ir",
     "whois.nic.ck",
     "whois.ra.net",
-    "whois.radb.net",
     NULL
 };
 
 const char *hide_strings[] = {
     "NOTICE AND TERMS OF USE: You", "",				/* NetSol */
-    "NOTICE: The expiration date", "reserves the right to",	/* crsnic */
+    "TERMS OF USE: You are not", "",				/* crsnic */
     "NOTICE: Access to .ORG WHOIS", "",
     "NOTICE: Access to .INFO WHOIS", "",
     "NOTICE: Access to the .aero", "",
@@ -60,6 +58,9 @@ const char *hide_strings[] = {
     "; This data is provided by domaindiscount24.com", "",
     "%% BookMyName Whois", "%% this policy",
     "The .coop registry WHOIS", "VERIFICATION, NOR DO",
+    "Tralliance, Inc., the Registry", "",			/* travel */
+    "NOTICE: Access to the domains information", "",		/* CORE */
+    "%% puntCAT Whois Server", "%% any time.",
     NULL, NULL
 };
 
